/* 
   uFS Project [ufs.h]
   (c) Simon Kueppers
   20.12.06 Germany

   This file is holds function prototypes for dir, file and disk management
   

   PS:
   I tried to comment the code as best as i was able to. This is a little bit confusing
   for a german, so please excuse if something is not that right it should be.
   Have fun with this code but remember the BEER LICENSE when you copy+paste something
   from here :-)

*/

#ifndef _ufs
#define _ufs

#include "stdint.h"									//Vartypedefs
#include "ufs_core.h"								//struct t_page


//////////////////////////////////////////////////////////////////////
//Function Prototypes & Defines for accessing directories
//////////////////////////////////////////////////////////////////////

/*
   Writes the content of directory 'diradress' on console by
   using the functions uart_puts and uart_putc

   path: directory to print
   return: 0 = not found, 1 = everything ok
*/
uint8_t ufs_printdir (uint8_t* path);

/*
   This function creates an empty (sub)directory in an alread existing
   directory pointed by 'diradress'. avoid trailing '/'. it will result in errors

   path: path to directory to create
   return: 0 = already existing or parent-dir to create new dir in does not exist. 1 = everything ok
*/
uint32_t ufs_makedir (uint8_t* path);

/*
   This function removes an empty(!) directory pointed by 'diradress'

   path: path to directory to delete
   return: 0 = dir does not exist. 1 = everything ok
*/
uint8_t ufs_remdir (uint8_t* path);





//////////////////////////////////////////////////////////////////////
//Function Prototypes & Defines for accessing files
//////////////////////////////////////////////////////////////////////

/*
   Defines for ufs_fseek ("origin")
*/
#define		FSEEK_START		0x55					//apply offset relative to start of file
#define		FSEEK_CUR		0x66					//apply offset relative to current position in file
#define		FSEEK_END		0x77					//apply file pointer to end of file

/*
   struct of filehandle
*/
struct t_file
{
	uint32_t pFileadress;							//Startadress of opened file
	uint32_t pCurrentPage;							//adress of actual page (points to headpage)
	uint16_t nCurrentByte;							//Actual # of byte in page
	struct t_page buffer;							//filebuffer
	void* pCurrentByte;								//Points to current byte in filebuffer
};


/*
   This function creates an empty file specified by 'path'

   path: path to file to create
   return: adress to created file
*/
uint32_t ufs_fcreate (uint8_t* path);

/*
   This function deletes a file specified by 'path'

   path: path to file to delete
   return: 0 = does not exist. 1 = ok
*/
uint8_t ufs_fdelete	(uint8_t* path);

/*
   This function copys a file from 'srcpath' to 'destpath'

   srcpath: path of sourcefile
   destpath: path of destfile
   return: 0 = srcpath not found or destpath already exists, 1 = ok
*/
uint8_t ufs_fcopy (uint8_t* srcpath, uint8_t* destpath);

/*
   gets size of file specified by 'path'

   path: path to file to get size from
   return: filesize in bytes
*/
uint32_t ufs_fgetsize (uint8_t* path);

/*
   opens a file specified by 'path' to a filehandle 'pHandle' with mode 'mode'

   pHandle: pointer to filehandle
   path: path to file to open
   mode: 'w' for writing, 'r' for reading, 'a' for appending
*/
uint8_t	ufs_fopen (struct t_file* pHandle, uint8_t* path, uint8_t mode);

/*
   Closes an opened file with handle 'pHandle'

   pHandle: pointer to filehandle
   return: no return value
*/
void ufs_fclose	(struct t_file* pHandle);

/*
   Puts a char to a file with filehandle 'pHandle' and increments filepointer by one

   pHandle: pointer to filehandle
   b: byte to write
   return: no return value
*/
void ufs_fputc (struct t_file* pHandle, uint8_t b);

/*
   Gets a char from file and increments filepointer by one

   pHandle: pointer to filehandle
   return: read byte
*/
uint8_t ufs_fgetc (struct t_file* pHandle);

/*
   Puts a '\0' terminated string to a file and increments filepointer by strlen(s)

   pHandle: pointer to filehandle
   return: no return value
*/
void ufs_fputs (struct t_file* pHandle, uint8_t* s);

/*
   This function reads 'nBytes' data from file to 'pBuf'

   pHandle: pointer to filehandle
   pBuf: pointer to buffer to read from
   nBytes: # of bytes to read
   return: 0 = feof occured, 1 = ok
*/
uint8_t ufs_fread (struct t_file* pHandle, void* pBuf, uint16_t nBytes);

/*
   This function writes 'nBytes' data from 'pBuf' to file

   pHandle: pointer to filehandle
   pBuf: pointer to buffer to write to
   nBytes: # of bytes to write
   return: 0 = disk full, 1 = ok
*/
uint8_t	ufs_fwrite (struct t_file* pHandle, void* pBuf, uint16_t nBytes);

/*
   This function re-positions the filepointer to an given offset 'offset'
   relative to 'origin'

   pHandle: pointer to filehandle
   offset: offset to set filepointer to
   origin: relative point in file to start from
   return: feof occured
*/
uint8_t ufs_fseek (struct t_file* pHandle, uint32_t offset, uint8_t origin);

/*
   Checks if next fgetX will lead to an eof. it checks if there are still bytes left to read.

   pHandle: pointer to filehandle
   return: 0 = no end of file, 1 = end of file
*/
uint8_t ufs_feof (struct t_file* pHandle);





//////////////////////////////////////////////////////////////////////
//Function Prototypes & Defines for accessing the disk
//////////////////////////////////////////////////////////////////////

/*
   This function returns the free space, that is available on disk

   return: free space on disk in byte
*/
uint32_t ufs_getfreespace();

/*
   This function returns the space on disk, that is used 

   return: used space on disk in byte

   NOTE: This function returns a multiple of 512. So the space, is not
   guaranteed as to be exactly. There is a small imprecision of maximal 512byte
*/
uint32_t ufs_getusedspace();

/*
   This function returns the size of the disk

   return: size of disk
*/
uint32_t ufs_getdisksize();

/*
   This function gets the disk name
   
   diskname: pointer to 255 char array used to hold the diskname
   return: equals to diskname
*/
uint8_t* ufs_getdiskname(uint8_t* diskname);




#endif //_ufs