/* 
   uFS Project [ufs_dirio.c]
   (c) Simon Kueppers
   20.12.06 Germany

   This file is only made for directory handling. Here you will find functions to 
   create, delete or print directorys.
  
   For function descriptions look at ufs_dirio.h
   

   PS:
   I tried to comment the code as best as i was able to. This is a little bit confusing
   for a german, so please excuse if something is not that right it should be.
   Have fun with this code but remember the BEER LICENSE when you copy+paste something
   from here :-)

*/

#include <stdlib.h>									//ultoa(), itoa()
#include <string.h>									//strcpy()
#include "stdint.h"									//Vartypedefs
#include "sd.h"										//function prototypes to access sd card
#include "time.h"									//function prototypes to real time clock
#include "uart.h"									//function prototypes to send data over uart
#include "ufs_core.h"								//function prototypes to ufs-core
#include "ufs.h"									//function prototypes and defines

/*
   Writes the content of directory 'diradress' on console by
   using the functions uart_puts and uart_putc
*/
uint8_t ufs_printdir(uint8_t* path)
{
	struct t_page TempPage;
	struct t_time TempTime;
	uint32_t currentpage;
	uint32_t diradress;
	uint32_t nDirs=0;
	uint32_t nFiles=0;
	uint16_t i;
	uint8_t strtemp[15];

	diradress = ufs_getdiradress(ROOTDIR_ADR, path);

	if (diradress == 0)								//simple check if dir exists
		return 0;

	sd_read(diradress, &TempPage);				//read this page to TempPage

	uart_puts("\nPath: ");
	uart_puts(path);								//and put some info on console
	
	uart_puts("\nCreated:  ");
	convtime(TempPage.headpage.tCreate, &TempTime);
	uart_puts(ultoa(TempTime.day, strtemp, 10));
	uart_putc('.');
	uart_puts(ultoa(TempTime.month, strtemp, 10));
	uart_putc('.');
	uart_puts(ultoa(TempTime.year, strtemp, 10));
	uart_putc(' ');
	uart_puts(ultoa(TempTime.hour, strtemp, 10));
	uart_putc(':');
	uart_puts(ultoa(TempTime.minute, strtemp, 10));
	uart_putc(':');
	uart_puts(ultoa(TempTime.second, strtemp, 10));

	uart_puts("\n\nContent:\nFlags| File/Dirname (Size) [Time of Creation]\n-----------------------------------------\n");


	while((currentpage = TempPage.hdr_pNextpage) != 0)//loop while not last page in chain
	{
		sd_read(currentpage, &TempPage);			//read next page to TempPage

		for(i=0; i<N_DIRENTRYS; i++)				//loop each dir-entry
		{
			if(TempPage.dirpage.entry[i])			//check if current entry valid
			{
				sd_read(TempPage.dirpage.entry[i], &TempPage);//fetch to buffer

				if (TempPage.headpage.attrflags & ATTR_ISDIR)//check if file or dir
				{
					uart_putc('D');					//if dir, print an "D" at the beginning
					nDirs++;						//and count the dirs-counter up by one
				}	else
				{
					uart_putc('F');					//if its a file, print an "F"
					nFiles++;						//and count the files-counter up by one
				}

				if (TempPage.headpage.attrflags & ATTR_ISPROT)//check if file/dir is protected
					uart_putc('R');
				else
					uart_putc(' ');


				if (TempPage.headpage.attrflags & ATTR_ISARCH)//check if file/dir is archived
					uart_putc('A');
				else
					uart_putc(' ');

				if (TempPage.headpage.attrflags & ATTR_ISHIDDEN)//check if file/dir is hidden
					uart_putc('H');
				else
					uart_putc(' ');

				if (TempPage.headpage.attrflags & ATTR_ISSYSTEM)//check if file/dir is system
					uart_putc('S');
				else
					uart_putc(' ');

				uart_puts("| ");
				uart_puts(TempPage.headpage.objname);//print file/dir name on console
				uart_puts(" (");					//print filesize on console
				uart_puts(ultoa(TempPage.headpage.filesize, strtemp, 10));
				uart_puts(" b) [");
	

				convtime(TempPage.headpage.tCreate, &TempTime);
				uart_puts(ultoa(TempTime.day, strtemp, 10));
				uart_putc('.');
				uart_puts(ultoa(TempTime.month, strtemp, 10));
				uart_putc('.');
				uart_puts(ultoa(TempTime.year, strtemp, 10));
				uart_putc(' ');
				uart_puts(ultoa(TempTime.hour, strtemp, 10));
				uart_putc(':');
				uart_puts(ultoa(TempTime.minute, strtemp, 10));
				uart_putc(':');
				uart_puts(ultoa(TempTime.second, strtemp, 10));
				
				uart_puts("]\n");

				sd_read(currentpage, &TempPage);	//and read dir-entry page back to TempPage
			}
		}
	}

	uart_putc('\n');
	uart_puts(itoa(nFiles, strtemp, 10));			//put some further stats on console
	uart_puts(" Files / ");
	uart_puts(itoa(nDirs, strtemp, 10));
	uart_puts(" Dirs\n");

	return 1;

}

/*
   This function creates an empty (sub)directory in an alread existing
   directory pointed by 'diradress'. avoid trailing '/'. it will result in errors
*/
uint32_t ufs_makedir(uint8_t* path)
{
	uint32_t diradress, linkadress;
	struct t_page TempPage;
	uint8_t* pObjname = 0;
	uint8_t* pPath = path;
	uint8_t c;

	if (*path == '/')
		pPath = ++path;

	while ((c = *path) != 0)						//Scan Filename
	{
		if (c == '/')								//for each '/' its containing (or not containing)
			pObjname = path;						//set pointer to last found '/' in string
		
		path++;
	}

	if (pObjname == 0)								//There was no '/' in string
	{
		diradress = ROOTDIR_ADR;					//new file will be created in root directory
		pObjname = pPath;							//desired filename begins at array-element 0

	}else
	if (pObjname == pPath)							//There is a '/' at the beginning
	{
		diradress = ROOTDIR_ADR;					//create new file in root directory
		pObjname++;									//next char is the beginning of the filename

	}else
	if (*(pObjname + 1) == '\0')					//the last found '/' is the last char in string
	{
		return 0;									//sorry, no filename provided. exit with error
	
	}else											//string contains path & filename
	{
		*pObjname = '\0';
	
		diradress = ufs_getdiradress(ROOTDIR_ADR, pPath);//get the directory adress of the scanned path

		if (diradress == 0)							//no (existent) directory
			return 0;								//return with error
		
		*pObjname = '/';							//replace string terminating zero with '/'
		pObjname++;									//pObjname now points to the beginning of the filename

	}

	
	if((linkadress = ufs_palloc()) == 0)			//allocate a new page
		return 0;									//and exist if not possible

	if (ufs_exists(diradress, pObjname))
		return 0;

	if (ufs_addentry(diradress, linkadress) == 0)	//add dir entry to scanned directory
	{
		ufs_pfree(linkadress);
		return 0;
	}

	TempPage.headpage.attrflags = ATTR_ISDIR;		//configure headpage
	TempPage.headpage.filesize = 0;
	TempPage.headpage.tCreate = gettime();
	TempPage.headpage.tLastaccess = 0;
	TempPage.headpage.tLastchange = 0;
	TempPage.headpage.pParent = diradress;
	strcpy(TempPage.headpage.objname, pObjname);
	
	TempPage.hdr_nValidbytes = 504;
	TempPage.hdr_pNextpage = 0;
		
	sd_write(linkadress, &TempPage);				//and write it to allocated space

	return linkadress;

}

/*
   This function removes an empty(!) directory pointed by 'diradress'
*/
uint8_t ufs_remdir(uint8_t* path)
{
	struct t_page TempPage;
	uint32_t diradress;

	diradress = ufs_getdiradress(ROOTDIR_ADR, path);

	if (diradress == 0) return 0;					//check if diradress is valid

	sd_read(diradress, &TempPage);					//yes, read it to TempPage

	if (TempPage.hdr_pNextpage) return 0;			//sorry, there are dir-entrys left in this directory, return error

	ufs_delentry(diradress);						//delete dir entry of directory
	ufs_pfree(diradress);							//free page

	return 1;										//return successful
}


	
